#!perl

# Massages the output of JavaCC.

$Parser = shift(@ARGV);
$TokenManager = shift(@ARGV);
$JJT = shift(@ARGV);

open(INPUT, $Parser);
open(OUTPUT, ">" . $Parser . "-temp");

$skipping_reinit = 0;

while (<INPUT>) {
    # Have the parser accept a string instead of a stream for the arg.
    # That way, we can assure that the "image" substrings come out of
    # a string we were going to keep around anyway, rather than a
    # newly consed string.
    if ($_ eq "  public JavaParser(java.io.InputStream stream) {\n") {
	print OUTPUT "  public JavaParser(String string) {\n";
    } elsif ($_ eq "  public void ReInit(java.io.InputStream stream) {\n") {
        $skipping_reinit = 1;
    } elsif ($_ eq "  public void ReInit(java.io.Reader stream) {\n") {
        $skipping_reinit = 1;
    } elsif ($_ eq "  public JavaParser(java.io.Reader stream) {\n") {
        $skipping_reinit = 1;
    } elsif ($skipping_reinit) {
        if ($_ =~ /\}/) {
	    $skipping_reinit = 0;
	}
    } elsif ($_ eq "    jj_input_stream = new ASCII_UCodeESC_CharStream(stream, 1, 1);\n") {
	print OUTPUT "    jj_input_stream = new ASCII_UCodeESC_CharStream(string, 1, 1);\n";
    }

    # normal case -- echo the input.
    else {
	print OUTPUT $_;
    }
}

close(INPUT);
close(OUTPUT);

open(INPUT, $TokenManager);
open(OUTPUT, ">" . $TokenManager . "-temp");

while (<INPUT>) {
    # rip out the stuff that sets endLine and endColumn, because it's gone.
    if (/t.endLine = / || /t.endColumn = /) {
	; # ignore the line

    # This next line is apparently both dead code and the cause of
    # the internal compiler error of Microsoft's JVC.  So output an 
    # empty statement instead.
    } elsif (/image\.append\(input_stream\.GetSuffix\(jjimageLen \+ \(lengthOfMatch = jjmatchedPos \+ 1\)\)\);/) {
	print OUTPUT "\t  ;\n";
    } else {
	print OUTPUT $_;
    }
}

close(INPUT);
close(OUTPUT);

# JJT generates code which spends amazing amounts of time and space creating
# Integer objects so that it can push them onto stacks.  This code transforms
# the file so that it does exactly the same thing, but faster.  This 
# transformation can be eliminated at any time without affecting the 
# correctness of the program.
open(INPUT, $JJT);
open(OUTPUT, ">" . $JJT . "-temp");

while (<INPUT>) {
    if (/java.util.Stack marks;/) {
	print OUTPUT "  private IntStack marks;\n";
    } elsif (/marks =/) {
	print OUTPUT "    marks = new IntStack();\n";
    } else {
	s/\(\(Integer\)marks.pop\(\)\).intValue\(\)/marks.pop()/;
	s/new Integer\((.*)\)\);/$1);/;
        print OUTPUT $_;
    }
}

close(INPUT);
close(OUTPUT);
